mod dns;
mod message;
mod server;

use server::*;
use message::{MessageQueue, MessageType};
use std::env;
use std::io::{Read, Write};
use std::time::Duration;
use std::thread;
use tun;


fn main() {


    let args: Vec<String> =env::args().collect();
    let port: u16 = 1053; // default port for the tunnel server

    // When used as server component
    if args.contains(&"--tunnel-server".to_string()) {

        // create a new tunnel server
        let mut server = DnsTunnel::new_server("swrzm.de".to_string(), port);
        server.serve();

    } else if args.contains(&"--tunnel-client".to_string()) {

        // connect to the tunnel server
        let tunnel = DnsTunnel::new("swrzm.de".to_string(), "192.168.122.1".to_string(), port);
        let poll_tunnel = tunnel.clone();

        tunnel.connect().expect("[-] Failed to connect to server");
        poll_tunnel.connect().expect("[-] Failed to connect to server");

        // Creating Tun interface with addr. 10.0.0.1
        let mut read_config = tun::Configuration::default();
        read_config.address((10, 0, 0, 2))
               .netmask((255, 255, 255, 0))
               .up();
        let mut write_config = tun::Configuration::default();
        write_config.address((10, 0, 0, 4))
               .netmask((255, 255, 255, 0))
               .up();

        let mut read_dev = tun::create(&read_config).unwrap();
        let mut write_dev = tun::create(&write_config).unwrap();
        let mut msg_queue = MessageQueue::new();

        // Create a new thread to handle the incoming packets, and resend ehm to the original
        // destination
        let polling_thread = thread::spawn(move || {
            let mut should_sleep = true;
            loop {
                if let Some(pkt) = poll_tunnel.poll() {
                    match pkt.msg_type {
                        MessageType::DATA => {
                            msg_queue.insert_msg(pkt);
                            should_sleep = false;
                        },
                        MessageType::DATAFIN => {
                            msg_queue.insert_msg(pkt);
                            should_sleep = true;
                        },
                        _ => {
                        }
                    }
                }

                for pkt in &msg_queue.packets {
                    write_dev.write(pkt).unwrap();
                }
                // clear the packets
                msg_queue.packets.clear();

                if should_sleep {
                    thread::sleep(Duration::from_millis(5));
                }
            }

        });
        let client_thread = thread::spawn(move || {
            println!("[+] Started client thread");
            // create a new thread for the tunnel
            let mut buf = [0; 4096];
            let mut amount: usize;
            loop {

                amount = read_dev.read(&mut buf).unwrap();
                if amount == 0 {
                    continue;
                }
                let packet = &buf[0 .. amount];
                /*
                match ip::Packet::new(packet) {
                    Ok(ip::Packet::V4(pkt)) => {
                        println!("[CLIENT -> SERVER] Captured packet: Src: {:?} Dst: {:?}", pkt.source(), pkt.destination());
                    }
                    _ => {
                    }
                }
                */
                tunnel.send_bytes(packet.to_vec());
            }
        });

        // Wait until both threads are finished
        client_thread.join().unwrap();
        polling_thread.join().unwrap();
    }
}
